/**
 * 
 * AiCiA - Android IRC Client
 * 
 * Copyright (C)2010 GORRY.
 * 
 */

package net.gorry.aicia;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.text.SpannableStringBuilder;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.Toast;

/**
 * 
 * IRCクライアント「AiCiA」
 * 
 * @author GORRY
 *
 */

public class ActivityMain extends Activity {
	private static final String TAG = "ActivityMain";
	private static final boolean V = false;

	/** */
	public static final int ACTIVITY_SYSTEMCONFIG = 1;

	private static Activity me;
	/** */
	private static IIRCService iIRCService;

	/** */
	public static String mCurrentServerName = null;
	/** */
	public static String mCurrentChannel = null;
	/** */
	public static String mTopic = null;

	/** */
	public static boolean mShutdownServiceOnDestroy = false;

	private final IRCServiceReceiver mReceiver = new IRCServiceReceiver();

	private String mInitOpenServerName = null;

	/** */
	public static Layout layout;
	/** */
	public static DoMain doMain;

	/** */
	public static boolean mDonate;

	/* アプリの一時退避
	 * @see android.app.Activity#onSaveInstanceState(android.os.Bundle)
	 */
	@Override
	protected void onSaveInstanceState(final Bundle outState) {
		if (V) Log.v(TAG, "onSaveInstanceState()");
		layout.saveInstanceState(outState);
	}

	/*
	 * アプリの一時退避復元
	 * @see android.app.Activity#onRestoreInstanceState(android.os.Bundle)
	 */
	@Override
	protected void onRestoreInstanceState(final Bundle savedInstanceState) {
		if (V) Log.v(TAG, "onRestoreInstanceState()");
		layout.restoreInstanceState(savedInstanceState);
	}

	/*
	 * 作成
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(final Bundle savedInstanceState) {
		if (V) Log.v(TAG, "onCreate()");
		super.onCreate(savedInstanceState);

		final Bundle extras = getIntent().getExtras();
		if (extras != null) {
			final boolean b = (extras.getBoolean("donate"));
			if (b) mDonate = true;
		}

		me = this;
		layout = new Layout(ActivityMain.this);
		doMain = new DoMain(ActivityMain.this);
		SystemConfig.setContext(ActivityMain.this);
		layout.setOrientation(true);
		SystemConfig.loadConfig();
		layout.setRotateMode();

		// getWindow().setSoftInputMode(LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
		layout.baseLayout_Create(true);

		if (iIRCService == null) {
			myBindService();
			myRegisterReceiver();
		} else {
			doMain.setIIRCService(iIRCService);
			layout.setIIRCService(iIRCService);
			layout.reviveLogs();
		}
		updateTitle();

		setContentView(layout.mBaseLayout);

	}

	/*
	 * 再起動
	 * @see android.app.Activity#onRestart()
	 */
	@Override
	public void onRestart() {
		if (V) Log.v(TAG, "onRestart()");
		super.onRestart();
	}

	/*
	 * 通知アイコンからの起動でIntentが付いてるとき
	 * @see android.app.Activity#onNewIntent(android.content.Intent)
	 */
	@Override
	public void onNewIntent(final Intent intent) {
		if (V) Log.v(TAG, "onNewIntent()");
		super.onNewIntent(intent);

		mInitOpenServerName = null;
		final Bundle extras = intent.getExtras();
		if (extras != null) {
			mInitOpenServerName = extras.getString("serverName");
		}
	}

	/*
	 * 開始
	 * @see android.app.Activity#onStart()
	 */
	@Override
	public void onStart() {
		if (V) Log.v(TAG, "onStart()");
		super.onStart();
	}

	/*
	 * 再開
	 * @see android.app.Activity#onResume()
	 */
	@Override
	public synchronized void onResume() {
		if (V) Log.v(TAG, "onResume()");
		super.onResume();
		new Thread(new Runnable() {	public void run() {
			if (iIRCService == null) {
				while (iIRCService == null) {
					try {
						Thread.sleep(200);
					} catch (final InterruptedException e) {
						//
					}
				}
				if (V) Log.v(TAG, "iIRCService loaded");
				doMain.setIIRCService(iIRCService);
				layout.setIIRCService(iIRCService);
			}
			layout.restoreInputBox();
			if (mInitOpenServerName != null) {
				try {
					final String serverName = mInitOpenServerName;
					final String channel = iIRCService.getCurrentChannel(serverName);
					mInitOpenServerName = null;
					if (channel != null) {
						(me).runOnUiThread(new Runnable(){ public void run() {
							doMain.doChangeChannel(serverName, channel, true);
						}});
					}
				} catch (final RemoteException e) {
					e.printStackTrace();
				}
			}
		} }).start();
	}

	/*
	 * 停止
	 * @see android.app.Activity#onPause()
	 */
	@Override
	public synchronized void onPause() {
		if (V) Log.v(TAG, "onPause()");
		super.onPause();
		layout.saveInputBox();
	}

	/*
	 * 中止
	 * @see android.app.Activity#onStop()
	 */
	@Override
	public void onStop() {
		if (V) Log.v(TAG, "onStop()");
		super.onStop();
	}

	//
	/*
	 * 破棄
	 * @see android.app.Activity#onDestroy()
	 */
	@Override
	public void onDestroy() {
		if (V) Log.v(TAG, "onDestroy()");
		super.onDestroy();
		unregisterReceiver(mReceiver);
		if (mShutdownServiceOnDestroy) {
			try {
				iIRCService.shutdown();
			}
			catch(final RemoteException e) {
				// エラー
			}
		}
		unbindService(ircServiceConn);
	}

	/*
	 * コンフィギュレーション変更
	 * @see android.app.Activity#onConfigurationChanged(android.content.res.Configuration)
	 */
	@Override
	public synchronized void onConfigurationChanged(final Configuration newConfig) {
		if (V) Log.v(TAG, "onConfigurationChanged()");
		super.onConfigurationChanged(newConfig);
		// 少し止めないと変更の反映が速すぎることがある
		try {
			Thread.sleep(500);
		} catch (final InterruptedException e) {
			//
		}
		layout.changeConfiguration(newConfig);
	}

	/*
	 * メインウィンドウにフォーカスが移るときの処理
	 * 子レイアウトを調整するためのエントリポイントとして使う
	 * @see android.app.Activity#onWindowFocusChanged(boolean)
	 */
	@Override
	public synchronized void onWindowFocusChanged(final boolean b) {
		if (V) Log.v(TAG, "onWindowFocusChanged()");
		layout.updateBaseLayout();
	}

	/**
	 * IRCサービスへのバインド
	 */
	private void myBindService() {
		final Intent i1 = new Intent(this, IRCService.class);
		i1.putExtra("donate", mDonate);
		startService(i1);
		final Intent i2 = new Intent(IIRCService.class.getName());
		i2.putExtra("donate", mDonate);
		bindService(i2, ircServiceConn, BIND_AUTO_CREATE);
	}

	/**
	 * IRCサービスのブロードキャストレシーバー登録
	 */
	private void myRegisterReceiver() {
		final IntentFilter filter = new IntentFilter(IRCService.ACTION);
		registerReceiver(mReceiver, filter);
	}

	/**
	 * IRCサービスからのブロードキャスト受信処理
	 */
	public class IRCServiceReceiver extends BroadcastReceiver {
		@Override
		public synchronized void onReceive(final Context context, final Intent intent) {
			if (V) Log.v(TAG, "onReceive()");
			final Bundle extras = intent.getExtras();
			final int msg = extras.getInt("msg");
			switch (msg) {
				case IRCService.RECEIVE_MESSAGE:
				{
					final String serverName = extras.getString("server");
					final String channel = extras.getString("channel");
					final String nick = extras.getString("nick");
					final String dateMsg = extras.getString("date");
					final SpannableStringBuilder ssb = new SpannableStringBuilder(extras.getCharSequence("ssb"));
					final SpannableStringBuilder ssbOther = new SpannableStringBuilder(extras.getCharSequence("ssbOther"));
					final Boolean alert = extras.getBoolean("alert");

					if (alert) {
						doAlert();
					}
					if ((mCurrentServerName != null) && (mCurrentChannel != null) &&
							serverName.equalsIgnoreCase(mCurrentServerName) &&
							channel.equalsIgnoreCase(mCurrentChannel)) {
						final SpannableStringBuilder ssbLine = IRCMsg.colorDateMsg(dateMsg, SystemConfig.mainLogDateColor);
						if ((nick != null) && (nick.length() > 0)) {
							final SpannableStringBuilder ssbNick = IRCMsg.colorNick(nick);
							ssbLine.append(ssbNick);
						}
						ssbLine.append(ssb);
						ssbLine.append("\n");
						layout.mMainLogWindow.addMessage(ssbLine);
						try {
							iIRCService.clearChannelUpdated(mCurrentServerName, mCurrentChannel);
						} catch (final RemoteException e) {
							e.printStackTrace();
						}
					} else {
						if (SystemConfig.now_subWindowMode > 0) {
							layout.mSubLogWindow.addMessage(ssbOther);
						}
					}
					break;
				}

				case IRCService.CREATE_SERVER:
				{
					final String serverName = extras.getString("server");
					@SuppressWarnings("unused")
					final String dateMsg = extras.getString("date");
					mCurrentServerName = serverName;
					mCurrentServerName = IRCMsg.sSystemChannelName;
					break;
				}

				case IRCService.REMOVE_SERVER:
				{
					break;
				}

				case IRCService.CREATE_CHANNEL:
				{
					final String serverName = extras.getString("server");
					final String channel = extras.getString("channel");
					@SuppressWarnings("unused")
					final String dateMsg = extras.getString("date");
					mCurrentServerName = serverName;
					mCurrentChannel = channel;
					try {
						mTopic = iIRCService.getTopic(serverName, channel);
					} catch (final RemoteException e) {
						e.printStackTrace();
					}
					updateTitle();
					layout.mMainLogWindow.clearLog();
					break;
				}

				case IRCService.REMOVE_CHANNEL:
				{
					break;
				}

				case IRCService.CHANGE_NICK:
				{
					break;
				}

				case IRCService.CHANGE_TOPIC:
				{
					final String serverName = extras.getString("server");
					final String channel = extras.getString("channel");
					final String topic = extras.getString("topic");
					@SuppressWarnings("unused")
					final String dateMsg = extras.getString("date");
					if ((mCurrentServerName != null) && (mCurrentChannel != null) &&
							serverName.equalsIgnoreCase(mCurrentServerName) &&
							channel.equalsIgnoreCase(mCurrentChannel)) {
						mTopic = topic;
						updateTitle();
					}
					break;
				}

				case IRCService.PING_PONG:
				{
					try {
						iIRCService.receivePong();
					} catch (final RemoteException e) {
						e.printStackTrace();
					}
					break;
				}

			}
		}
	}

	/**
	 * IRCサービスのAPI取得処理
	 */
	private final ServiceConnection ircServiceConn = new ServiceConnection() {
		public void onServiceConnected(final ComponentName name, final IBinder service) {
			if (V) Log.v(TAG, "onServiceConnected()");
			final IIRCService i = IIRCService.Stub.asInterface(service);
			if (V) Log.v(TAG, "iIRCService loaded");
			doMain.setIIRCService(i);
			layout.setIIRCService(i);
			iIRCService = i;
			layout.reviveLogs();
		}
		public void onServiceDisconnected(final ComponentName name) {
			if (V) Log.v(TAG, "onServiceDisconnected()");
			iIRCService = null;
			// layout.setIIRCService(iIRCService);
		}
	};

	/*
	 * オプションメニュー作成
	 * @see android.app.Activity#onCreateOptionsMenu(android.view.Menu)
	 */
	@Override
	public boolean onCreateOptionsMenu(final Menu menu) {
		if (V) Log.v(TAG, "onCreateOptionMenu()");
		final MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.menu_main, menu);
		return true;
	}

	/*
	 * オプションメニュー選択処理
	 * @see android.app.Activity#onOptionsItemSelected(android.view.MenuItem)
	 */
	@Override
	public boolean onOptionsItemSelected(final MenuItem item) {
		if (V) Log.v(TAG, "onOptionsItemSelected()");
		switch (item.getItemId()) {
			case R.id.serverlist:
			{
				doMain.doServerListConfig();
				return true;
			}

			case R.id.help:
			{
				doMain.doShowHelp();
				return true;
			}

			case R.id.version:
			{
				doShowVersion();
				return true;
			}

			case R.id.connect:
			{
				doMain.doConnectAuto();
				return true;
			}

			case R.id.shutdown:
			{
				doMain.doShutdown();
				return true;
			}

			case R.id.setting:
			{
				doMain.doSystemConfig();
				return true;
			}
		}
		return false;
	}

	/*
	 * アクティビティの結果処理
	 * @see android.app.Activity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	protected void onActivityResult(final int requestCode, final int resultCode, final Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		switch (requestCode) {
			case ACTIVITY_SYSTEMCONFIG:
				SystemConfig.loadConfig();
				int rebootLevel = 0;
				if (data != null) {
					final Bundle extras = data.getExtras();
					if (extras != null) {
						rebootLevel = extras.getInt("rebootlevel");
					}
				}
				if ((rebootLevel & 2) != 0) {
					try {
						iIRCService.reloadSystemConfig();
					} catch (final RemoteException e) {
						e.printStackTrace();
					}
				}
				if ((rebootLevel & 1) != 0) {
					layout.rebootLayout();
				}

				break;
		}

	}

	/**
	 * Short Toastを表示
	 * @param message メッセージ
	 */
	public void myShortToastShow(final String message) {
		(me).runOnUiThread(new Runnable(){ public void run() {
			final Toast toast = Toast.makeText(me, message, Toast.LENGTH_SHORT);
			toast.show();
		} });
	}

	/**
	 * タイトルバーの更新
	 */
	public void updateTitle() {
		if ((mCurrentChannel != null) && (mCurrentServerName != null)) {
			setTitle(IRCMsg.serverChannelTopicTitle(mCurrentServerName, mCurrentChannel, mTopic));
		} else {
			setTitle((mDonate ? "AiCiA (DONATED)" : "AiCiA"));
		}
	}

	/**
	 * アラート処理
	 */
	private void doAlert() {
		// final Vibrator vib = (Vibrator)getSystemService(VIBRATOR_SERVICE);
		// final long[] pat = {200,200,200,200,200,200,200,200};
		// vib.vibrate(pat, -1);
	}

	/**
	 * バージョン表示
	 */
	public void doShowVersion() {
		if (V) Log.v(TAG, "doShowVersion()");
		String versionName = null;
		final PackageManager pm = getPackageManager();
		try {
			PackageInfo info = null;
			if (ActivityMain.mDonate) {
				info = pm.getPackageInfo("net.gorry.aicia_donate", 0);
			} else {
				info = pm.getPackageInfo("net.gorry.aicia", 0);
			}
			versionName = info.versionName;
		} catch (final NameNotFoundException e) {
			e.printStackTrace();
		}
		final AlertDialog.Builder bldr = new AlertDialog.Builder(me);
		if (ActivityMain.mDonate) {
			bldr.setTitle("AiCiA - Android IRC Client\n(DONATED)");
		} else {
			bldr.setTitle("AiCiA - Android IRC Client");
		}
		bldr.setMessage("Version " + versionName + "\nCopyright (C)2010 GORRY.")
		.setIcon(R.drawable.icon);
		bldr.create().show();
	}

}


